@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,7 +23,8 @@ the scope, to make sure it can be used without.
 
 Note: This is using "optparse", because "argparse" is only Python 2.7 and
 higher, and we still support Python 2.6 due to the RHELs still being used,
-and despite the long deprecation, it's in every later release.
+and despite the long deprecation, it's in every later release, and actually
+pretty good.
 """
 
 import os
@@ -36,6 +37,7 @@ from nuitka.PythonFlavors import (
     isApplePython,
     isCPythonOfficialPackage,
     isDebianPackagePython,
+    isFedoraPackagePython,
     isHomebrewPython,
     isMSYS2MingwPython,
     isNuitkaPython,
@@ -50,6 +52,7 @@ from nuitka.utils.Utils import (
     getWindowsRelease,
     isLinux,
     isPosixWindows,
+    isWin32OrPosixWindows,
     withNoSyntaxWarning,
 )
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
@@ -75,6 +78,8 @@ def _getPythonFlavor():
         return "WinPython"
     elif isDebianPackagePython():
         return "Debian Python"
+    elif isFedoraPackagePython():
+        return "Fedora Python"
     elif isHomebrewPython():
         return "Homebrew Python"
     elif isApplePython():
@@ -114,7 +119,7 @@ def _getVersionInformationValues():
         else:
             yield "Distribution: %s %s" % (dist_name, dist_version)
 
-    if getOS() == "Windows":
+    if isWin32OrPosixWindows():
         yield "WindowsRelease: %s" % getWindowsRelease()
 
 
@@ -208,8 +213,8 @@ Python flags to use. Default is what you are using to run Nuitka, this
 enforces a specific mode. These are options that also exist to standard
 Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
-give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
-(do not use docstrings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
+give Python run time warnings), "-O" (alias "no_asserts"), "no_docstrings"
+(do not use doc strings), "-u" (alias "unbuffered") and "-m".  Default empty.""",
 )
 
 parser.add_option(
@@ -220,42 +225,14 @@ parser.add_option(
     default=None,
     help="""\
 If using Python3.3 or Python3.4, provide the path of a Python binary to use
-for Scons. Otherwise Nuitka can use what you run Nuitka with or a "scons"
-binary that is found in PATH, or a Python installation from Windows registry.""",
-)
-
-parser.add_option(
-    "--warn-implicit-exceptions",
-    action="store_true",
-    dest="warn_implicit_exceptions",
-    default=False,
-    help="""\
-Enable warnings for implicit exceptions detected at compile time.""",
-)
-
-parser.add_option(
-    "--warn-unusual-code",
-    action="store_true",
-    dest="warn_unusual_code",
-    default=False,
-    help="""\
-Enable warnings for unusual code detected at compile time.""",
-)
-
-parser.add_option(
-    "--assume-yes-for-downloads",
-    action="store_true",
-    dest="assume_yes_for_downloads",
-    default=False,
-    help="""\
-Allow Nuitka to download external code if necessary, e.g. dependency
-walker, ccache, and even gcc on Windows. To disable, redirect input
-from nul device, e.g. "</dev/null" or "<NUL:". Default is to prompt.""",
+for Scons. Otherwise Nuitka can use what you run Nuitka with or a Python
+installation from Windows registry. On Windows Python 3.5 or higher is
+needed. On non-Windows, Python 2.6 or 2.7 will do as well.""",
 )
 
 
 include_group = OptionGroup(
-    parser, "Control the inclusion of modules and packages in result."
+    parser, "Control the inclusion of modules and packages in result"
 )
 
 include_group.add_option(
@@ -291,9 +268,12 @@ include_group.add_option(
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
-Include the content of that directory, no matter if it's used by the given main
-program in a visible form. Overrides all other inclusion options. Can be given
-multiple times. Default empty.""",
+Include also the code found in that directory, considering as if
+they are each given as a main file. Overrides all other inclusion
+options. You ought to prefer other inclusion options, that go by
+names, rather than filenames, those find things through being in
+"sys.path". This option is for very special use cases only. Can
+be given multiple times. Default empty.""",
 )
 
 include_group.add_option(
@@ -327,11 +307,10 @@ include_group.add_option(
     help=SUPPRESS_HELP,
 )
 
-
 parser.add_option_group(include_group)
 
-follow_group = OptionGroup(parser, "Control the following into imported modules")
 
+follow_group = OptionGroup(parser, "Control the following into imported modules")
 
 follow_group.add_option(
     "--follow-stdlib",
@@ -388,8 +367,25 @@ times. Default empty.""",
 
 parser.add_option_group(follow_group)
 
+onefile_group = OptionGroup(parser, "Onefile options")
 
-data_group = OptionGroup(parser, "Data files for standalone/onefile mode")
+onefile_group.add_option(
+    "--onefile-tempdir-spec",
+    action="store",
+    dest="onefile_tempdir_spec",
+    metavar="ONEFILE_TEMPDIR_SPEC",
+    default=None,
+    help="""\
+Use this as a folder to unpack to in onefile mode. Defaults to
+'%TEMP%/onefile_%PID%_%TIME%', i.e. user temporary directory
+and being non-static it's removed. Use e.g. a string like
+'%CACHE_DIR%/%COMPANY%/%PRODUCT%/%VERSION%' which is a good
+static cache path, this will then not be removed.""",
+)
+
+parser.add_option_group(onefile_group)
+
+data_group = OptionGroup(parser, "Data files")
 
 data_group.add_option(
     "--include-package-data",
@@ -398,10 +394,19 @@ data_group.add_option(
     metavar="PACKAGE",
     default=[],
     help="""\
-Include data files of the given package name. Can use patterns. By default
-Nuitka does not unless hard coded and vital for operation of a package. This
-will include all non-DLL, non-extension modules in the distribution. Default
-empty.""",
+Include data files for the given package name. DLLs and extension modules
+are not data files and never included like this. Can use patterns for
+module names and patterns for the filenames below. By default Nuitka
+does not include them, unless through configuration due to being vital
+for operation of a package. With a "*" pattern for package name, you
+can change that, but will include way too many files.
+This will only include non-DLL, non-extension modules in the package
+directory. After a ":" optionally a filename pattern can be given as
+well, selecting only matching files. Examples:
+"--include-package-data=package_name" (all files)
+"--include-package-data=package_name=*.txt" (only certain type)
+"--include-package-data=package_name=some_filename.dat (concrete file)
+Default empty.""",
 )
 
 data_group.add_option(
@@ -429,9 +434,9 @@ data_group.add_option(
     help="""\
 Include data files from complete directory in the distribution. This is
 recursive. Check '--include-data-files' with patterns if you want non-recursive
-inclusion. An example would be '--include-data-dir=/path/somedir=data/somedir'
+inclusion. An example would be '--include-data-dir=/path/some_dir=data/some_dir'
 for plain copy, of the whole directory. All files are copied, if you want to
-exclude files you need to remove them beforehand, or use --noinclude-data-files
+exclude files you need to remove them beforehand, or use '--noinclude-data-files'
 option to remove them. Default empty.""",
 )
 
@@ -451,6 +456,72 @@ empty.""",
 parser.add_option_group(data_group)
 
 
+dll_group = OptionGroup(parser, "DLL files")
+
+dll_group.add_option(
+    "--noinclude-dlls",
+    action="append",
+    dest="dll_files_inhibited",
+    metavar="PATTERN",
+    default=[],
+    help="""\
+Do not include DLL files matching the filename pattern given. This is against
+the target filename, not source paths. So ignore a DLL "someDLL" contained in
+the package "package_name" it should be matched as "package_name/someDLL.*".
+Default empty.""",
+)
+
+parser.add_option_group(dll_group)
+
+warnings_group = OptionGroup(parser, "Control the warnings to be given by Nuitka")
+
+
+warnings_group.add_option(
+    "--warn-implicit-exceptions",
+    action="store_true",
+    dest="warn_implicit_exceptions",
+    default=False,
+    help="""\
+Enable warnings for implicit exceptions detected at compile time.""",
+)
+
+warnings_group.add_option(
+    "--warn-unusual-code",
+    action="store_true",
+    dest="warn_unusual_code",
+    default=False,
+    help="""\
+Enable warnings for unusual code detected at compile time.""",
+)
+
+warnings_group.add_option(
+    "--assume-yes-for-downloads",
+    action="store_true",
+    dest="assume_yes_for_downloads",
+    default=False,
+    help="""\
+Allow Nuitka to download external code if necessary, e.g. dependency
+walker, ccache, and even gcc on Windows. To disable, redirect input
+from nul device, e.g. "</dev/null" or "<NUL:". Default is to prompt.""",
+)
+
+
+warnings_group.add_option(
+    "--nowarn-mnemonic",
+    action="append",
+    dest="nowarn_mnemonics",
+    metavar="MNEMONIC",
+    default=[],
+    help="""\
+Disable warning for a given mnemonic. These are given to make sure you are aware of
+certain topics, and typically point to the Nuitka website. The mnemonic is the part
+of the URL at the end, without the HTML suffix. Can be given multiple times and
+accepts shell pattern. Default empty.""",
+)
+
+parser.add_option_group(warnings_group)
+
+
 execute_group = OptionGroup(parser, "Immediate execution after compilation")
 
 execute_group.add_option(
@@ -481,30 +552,45 @@ execute_group.add_option(
     dest="keep_pythonpath",
     default=False,
     help="""\
-When immediately executing the created binary (--execute), don't reset
-PYTHONPATH. When all modules are successfully included, you ought to not need
-PYTHONPATH anymore.""",
+When immediately executing the created binary ('--execute'), don't reset
+'PYTHONPATH' environment. When all modules are successfully included, you
+ought to not need PYTHONPATH anymore.""",
 )
 
 parser.add_option_group(execute_group)
 
+
 dump_group = OptionGroup(parser, "Dump options for internal tree")
 
 dump_group.add_option(
     "--xml",
-    action="store_true",
-    dest="dump_xml",
-    default=False,
-    help="Dump the final result of optimization as XML, then exit.",
+    action="store",
+    dest="xml_output",
+    metavar="XML_OUTPUT",
+    default=None,
+    help="Write the final result of optimization in XML form to given filename.",
 )
 
 
 parser.add_option_group(dump_group)
 
 
-codegen_group = OptionGroup(parser, "Code generation choices")
+compilation_group = OptionGroup(parser, "Compilation choices")
 
-codegen_group.add_option(
+compilation_group.add_option(
+    "--user-package-configuration-file",
+    action="append",
+    dest="user_yaml_files",
+    default=[],
+    metavar="USER_YAML",
+    help="""\
+User provided Yaml file with package configuration. You can include DLLs,
+remove bloat, add hidden dependencies. Check User Manual for a complete
+description of the format to use. Can be given multiple times. Defaults
+to empty.""",
+)
+
+compilation_group.add_option(
     "--disable-bytecode-cache",
     action="store_true",
     dest="disable_bytecode_cache",
@@ -514,7 +600,7 @@ Do not reuse dependency analysis results for modules, esp. from standard library
 that are included as bytecode.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--full-compat",
     action="store_false",
     dest="improved",
@@ -523,11 +609,11 @@ codegen_group.add_option(
 Enforce absolute compatibility with CPython. Do not even allow minor
 deviations from CPython behavior, e.g. not having better tracebacks
 or exception messages which are not really incompatible, but only
-different. This is intended for tests only and should not be used
-for normal use.""",
+different or worse. This is intended for tests only and should *not*
+be used.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--file-reference-choice",
     action="store",
     dest="file_reference_mode",
@@ -546,7 +632,7 @@ compatibility reasons, the "__file__" value will always have ".py" suffix
 independent of what it really is.""",
 )
 
-codegen_group.add_option(
+compilation_group.add_option(
     "--module-name-choice",
     action="store",
     dest="module_name_mode",
@@ -562,11 +648,13 @@ is incompatible for modules that normally can be loaded into any package.""",
 )
 
 
-parser.add_option_group(codegen_group)
+parser.add_option_group(compilation_group)
+
 
 output_group = OptionGroup(parser, "Output choices")
 
 output_group.add_option(
+    "--output-filename",
     "-o",
     action="store",
     dest="output_filename",
@@ -579,7 +667,7 @@ include path information that needs to exist though. Defaults to '%s' on this
 platform.
 """
     % "<program_name>"
-    + (".exe" if getOS() == "Windows" else ".bin"),
+    + (".exe" if isWin32OrPosixWindows() else ".bin"),
 )
 
 output_group.add_option(
@@ -590,7 +678,7 @@ output_group.add_option(
     default="",
     help="""\
 Specify where intermediate and final output files should be put. The DIRECTORY
-will be populated with C files, object files, etc.
+will be populated with build folder, dist folder, binaries, etc.
 Defaults to current directory.
 """,
 )
@@ -632,9 +720,10 @@ production. Defaults to off.""",
 )
 
 debug_group.add_option(
+    "--unstripped",
     "--unstriped",
     action="store_true",
-    dest="unstriped",
+    dest="unstripped",
     default=False,
     help="""\
 Keep debug info in the resulting object file for better debugger interaction.
@@ -763,8 +852,8 @@ parser.add_option(
 
 parser.add_option_group(debug_group)
 
-c_compiler_group = OptionGroup(parser, "Backend C compiler choice")
 
+c_compiler_group = OptionGroup(parser, "Backend C compiler choice")
 
 c_compiler_group.add_option(
     "--clang",
@@ -847,6 +936,7 @@ Do not attempt to use ccache (gcc, clang, etc.) or clcache (MSVC, clangcl).""",
 
 parser.add_option_group(c_compiler_group)
 
+
 pgo_group = OptionGroup(parser, "PGO compilation choices")
 
 pgo_group.add_option(
@@ -909,6 +999,7 @@ launch it through a script that prepares it to run. Default use created program.
 
 parser.add_option_group(pgo_group)
 
+
 tracing_group = OptionGroup(parser, "Tracing features")
 
 tracing_group.add_option(
@@ -945,8 +1036,7 @@ tracing_group.add_option(
     action="store_false",
     dest="progress_bar",
     default=True,
-    help="""Disable progress bar outputs (if tqdm is installed).
-Defaults to off.""",
+    help="""Disable progress bar. Defaults to off.""",
 )
 
 
@@ -986,7 +1076,7 @@ tracing_group.add_option(
     dest="compilation_report_filename",
     default=None,
     help="""\
-Report module inclusion in an XML output file. Default is off.""",
+Report module, data file, compilation details in an XML output file. Default is off.""",
 )
 
 tracing_group.add_option(
@@ -1014,6 +1104,29 @@ parser.add_option_group(tracing_group)
 
 os_group = OptionGroup(parser, "General OS controls")
 
+os_group.add_option(
+    "--disable-console",
+    "--macos-disable-console",
+    "--windows-disable-console",
+    action="store_true",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, disable the console window and create a GUI
+application. Defaults to off.""",
+)
+
+os_group.add_option(
+    "--enable-console",
+    action="store_false",
+    dest="disable_console",
+    default=None,
+    help="""\
+When compiling for Windows or macOS, enable the console window and create a console
+application. This disables hints from certain modules, e.g. "PySide" that suggest
+to disable it. Defaults to true.""",
+)
+
 os_group.add_option(
     "--force-stdout-spec",
     "--windows-force-stdout-spec",
@@ -1043,6 +1156,7 @@ Defaults to not active, use e.g. '%PROGRAM%.err.txt', i.e. file near your progra
 
 parser.add_option_group(os_group)
 
+
 windows_group = OptionGroup(parser, "Windows specific controls")
 
 windows_group.add_option(
@@ -1053,15 +1167,6 @@ windows_group.add_option(
     help=SUPPRESS_HELP,
 )
 
-windows_group.add_option(
-    "--windows-disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
-    help="""\
-When compiling for Windows, disable the console window. Defaults to off.""",
-)
-
 windows_group.add_option(
     "--windows-icon-from-ico",
     action="append",
@@ -1103,7 +1208,7 @@ windows_group.add_option(
 )
 
 windows_group.add_option(
-    "--windows-uac-uiaccess",
+    "--windows-uac-uiaccess",  # spell-checker: ignore uiaccess
     action="store_true",
     dest="windows_uac_uiaccess",
     metavar="WINDOWS_UAC_UIACCESS",
@@ -1114,6 +1219,7 @@ desktop access. (Windows only). Defaults to off.""",
 )
 
 windows_group.add_option(
+    "--company-name",
     "--windows-company-name",
     action="store",
     dest="windows_company_name",
@@ -1127,6 +1233,7 @@ added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
 
 windows_group.add_option(
+    "--product-name",
     "--windows-product-name",
     action="store",
     dest="windows_product_name",
@@ -1140,6 +1247,7 @@ filename of the binary.""",
 
 windows_group.add_option(
     "--windows-file-version",
+    "--file-version",
     action="store",
     dest="windows_file_version",
     metavar="WINDOWS_FILE_VERSION",
@@ -1152,6 +1260,7 @@ added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
 
 windows_group.add_option(
+    "--product-version",
     "--windows-product-version",
     action="store",
     dest="windows_product_version",
@@ -1177,29 +1286,9 @@ One of file or product version is required, when a version resource needs to be
 added, e.g. to specify product name, or company name. Defaults to nonsense.""",
 )
 
-windows_group.add_option(
-    "--windows-onefile-tempdir",
-    "--onefile-tempdir",
-    action="store_true",
-    dest="is_onefile_tempdir",
-    metavar="ONEFILE_TEMPDIR",
-    default=False,
-    help=SUPPRESS_HELP,
-)
-
-windows_group.add_option(
-    "--windows-onefile-tempdir-spec",
-    "--onefile-tempdir-spec",
-    action="store",
-    dest="onefile_tempdir_spec",
-    metavar="ONEFILE_TEMPDIR_SPEC",
-    default=None,
-    help="""\
-Use this as a temporary folder. Defaults to '%TEMP%\\onefile_%PID%_%TIME%', i.e. system temporary directory.""",
-)
-
 parser.add_option_group(windows_group)
 
+
 macos_group = OptionGroup(parser, "macOS specific controls")
 
 macos_group.add_option(
@@ -1215,17 +1304,6 @@ is what the running Python allows for. Default is "native" which is
 the architecture the Python is run with.""",
 )
 
-macos_group.add_option(
-    "--macos-disable-console",
-    "--disable-console",
-    action="store_true",
-    dest="disable_console",
-    default=False,
-    help="""\
-When compiling for macOS, disable the console window and create a GUI
-application. Defaults to off.""",
-)
-
 macos_group.add_option(
     "--macos-create-app-bundle",
     action="store_true",
@@ -1254,7 +1332,7 @@ macos_group.add_option(
     metavar="MACOS_SIGNED_APP_NAME",
     default=None,
     help="""\
-Name of the application to use for macOS signing. Follow "com.yourcompany.appname"
+Name of the application to use for macOS signing. Follow "com.YourCompany.AppName"
 naming results for best results, as these have to be globally unique, and will
 potentially grant protected API accesses.""",
 )
@@ -1275,14 +1353,24 @@ macos_group.add_option(
     action="store",
     dest="macos_sign_identity",
     metavar="MACOS_APP_VERSION",
-    default="-",
+    default="ad-hoc",
     help="""\
 When signing on macOS, by default an ad-hoc identify will be used, but with this
 option your get to specify another identity to use. The signing of code is now
-mandatory on macOS and cannot be disabled. Default "-" if not given, which means
-ad-hoc.""",
+mandatory on macOS and cannot be disabled. Default "ad-hoc" if not given.""",
 )
 
+macos_group.add_option(
+    "--macos-sign-notarization",
+    action="store_true",
+    dest="macos_sign_notarization",
+    default=False,
+    help="""\
+When signing for notarization, using a proper TeamID identity from Apple, use
+the required runtime signing option, such that it can be accepted.""",
+)
+
+
 macos_group.add_option(
     "--macos-app-version",
     action="store",
@@ -1313,9 +1401,11 @@ the option can be specified multiple times. Default empty.""",
 
 parser.add_option_group(macos_group)
 
+
 linux_group = OptionGroup(parser, "Linux specific controls")
 
 linux_group.add_option(
+    "--linux-icon",
     "--linux-onefile-icon",
     action="append",
     dest="icon_path",
@@ -1324,18 +1414,9 @@ linux_group.add_option(
     help="Add executable icon for onefile binary to use. Can be given only one time. Defaults to Python icon if available.",
 )
 
-linux_group.add_option(
-    "--linux-onefile-compression",
-    action="store",
-    dest="appimage_compression",
-    choices=("gzip", "xz"),
-    metavar="COMPRESSION",
-    default="gzip",
-    help="Compression method to use for Linux onefile builds. Defaults to gzip for faster decompression",
-)
-
 parser.add_option_group(linux_group)
 
+
 plugin_group = OptionGroup(parser, "Plugin control")
 
 plugin_group.add_option(
@@ -1384,9 +1465,6 @@ plugin_group.add_option(
 Show list of all available plugins and exit. Defaults to off.""",
 )
 
-
-parser.add_option_group(plugin_group)
-
 plugin_group.add_option(
     "--user-plugin",
     action="append",
@@ -1406,16 +1484,18 @@ Show source changes to original Python file content before compilation. Mostly
 intended for developing plugins. Default False.""",
 )
 
+parser.add_option_group(plugin_group)
+
 
 def _considerPluginOptions(logger):
     # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
-        addStandardPluginCommandlineOptions,
+        addStandardPluginCommandLineOptions,
         addUserPluginCommandLineOptions,
     )
 
-    addStandardPluginCommandlineOptions(parser=parser)
+    addStandardPluginCommandLineOptions(parser=parser)
 
     for arg in sys.argv[1:]:
         if arg.startswith(("--enable-plugin=", "--plugin-enable=")):
@@ -1467,7 +1547,7 @@ def _expandProjectArg(arg, filename_arg, for_eval):
     values["Linux_Distribution_Base"] = dist_info[1] or dist_info[0]
     values["Linux_Distribution_Version"] = dist_info[2]
 
-    if getOS() == "Windows":
+    if isWin32OrPosixWindows():
         values["WindowsRelease"] = getWindowsRelease()
 
     arg = arg.format(**values)
@@ -1499,7 +1579,7 @@ def _getProjectOptions(logger, filename_arg, module_mode):
     cond_level = -1
 
     for line_number, line in enumerate(contents_by_line):
-        match = re.match(b"^\\s*#(\\s+)nuitka-project(.*?):(.*)", line)
+        match = re.match(b"^\\s*#(\\s*)nuitka-project(.*?):(.*)", line)
 
         if match:
             level, command, arg = match.groups()
@@ -1515,7 +1595,7 @@ def _getProjectOptions(logger, filename_arg, module_mode):
 
             expect_block = False
 
-            if level == cond_level and command == "-else":
+            if level == cond_level and command == b"-else":
                 execute_block = not execute_block
             elif level <= cond_level:
                 execute_block = True
@@ -1534,7 +1614,7 @@ def _getProjectOptions(logger, filename_arg, module_mode):
                         "Error, 'nuitka-project-if' needs to start a block with a colon at line end.",
                     )
 
-                arg = arg[:-1]
+                arg = arg[:-1].strip()
 
                 expanded = _expandProjectArg(arg, filename_arg, for_eval=True)
 
@@ -1546,7 +1626,7 @@ def _getProjectOptions(logger, filename_arg, module_mode):
                 # Likely mistakes, e.g. with "in" tests.
                 if r is not True and r is not False:
                     sys.exit(
-                        "Error, 'nuitka-project-if' condition %r (%r) does not yield boolean result %r"
+                        "Error, 'nuitka-project-if' condition %r (expanded to %r) does not yield boolean result %r"
                         % (arg, expanded, r)
                     )
 
