@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -62,7 +62,7 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
     def getSimulator(self):
         return PythonOperators.all_comparison_functions[self.comparator]
 
-    def _computeCompileTimeConstantComparision(self, trace_collection):
+    def _computeCompileTimeConstantComparison(self, trace_collection):
         left_value = self.subnode_left.getCompileTimeConstant()
         right_value = self.subnode_right.getCompileTimeConstant()
 
@@ -72,7 +72,7 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
             description="Comparison of constant arguments.",
         )
 
-    def makeInverseComparision(self):
+    def makeInverseComparison(self):
         # Making this accessing for tree building phase as well.
         return makeComparisonExpression(
             left=self.subnode_left,
@@ -83,7 +83,7 @@ class ExpressionComparisonBase(ExpressionChildrenHavingBase):
 
     def computeExpressionOperationNot(self, not_node, trace_collection):
         if self.getTypeShape() is tshape_bool:
-            result = self.makeInverseComparision()
+            result = self.makeInverseComparison()
 
             result.copyTraceStateFrom(self)
 
@@ -512,10 +512,11 @@ class ExpressionComparisonExceptionMatchBase(
                     description="Exception matched with constant arguments.",
                 )
 
-        # Any code could be run, note that.
-        trace_collection.onControlFlowEscape(self)
+        if self.mayRaiseExceptionComparison():
+            # Any code could be run, note that.
+            trace_collection.onControlFlowEscape(self)
 
-        trace_collection.onExceptionRaiseExit(BaseException)
+            trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
 
@@ -537,7 +538,6 @@ class ExpressionComparisonExceptionMatchBase(
         if python_version < 0x300:
             return False
 
-        # TODO: Add shape for exceptions.
         type_shape = self.subnode_right.getTypeShape()
 
         if type_shape is tshape_exception_class:
@@ -561,6 +561,13 @@ class ExpressionComparisonExceptionMismatch(ExpressionComparisonExceptionMatchBa
 class ExpressionComparisonInNotInBase(
     ExpressionBoolShapeExactMixin, ExpressionComparisonBase
 ):
+    __slots__ = (
+        "left_available",
+        "left_comparable",
+        "right_available",
+        "right_comparable",
+    )
+
     def __init__(self, left, right, source_ref):
         ExpressionComparisonBase.__init__(
             self, left=left, right=right, source_ref=source_ref
@@ -568,6 +575,11 @@ class ExpressionComparisonInNotInBase(
 
         assert self.comparator in ("In", "NotIn")
 
+        self.left_available = False
+        self.left_comparable = None
+        self.right_available = False
+        self.right_comparable = None
+
     @staticmethod
     def getDetails():
         return {}
@@ -585,7 +597,33 @@ class ExpressionComparisonInNotInBase(
 
         return right.mayRaiseExceptionIn(exception_type, left)
 
+    def getSimulator(self):
+        return PythonOperators.other_comparison_functions[self.comparator]
+
     def computeExpression(self, trace_collection):
+        if not self.left_available:
+            (
+                self.left_available,
+                self.left_comparable,
+            ) = self.subnode_left.getComparisonValue()
+
+        if self.left_available:
+            if not self.right_available:
+                (
+                    self.right_available,
+                    self.right_comparable,
+                ) = self.subnode_right.getComparisonValue()
+
+            if self.right_available:
+                return trace_collection.getCompileTimeComputationResult(
+                    node=self,
+                    computation=lambda: self.getSimulator()(
+                        self.left_comparable, self.right_comparable
+                    ),
+                    description="Contains check %s of constant arguments."
+                    % self.comparator,
+                )
+
         return self.subnode_right.computeExpressionComparisonIn(
             in_node=self,
             value_node=self.subnode_left,
