@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,6 @@ to eliminate or limit their impact as much as possible, but it's difficult
 to do.
 """
 
-from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
 from nuitka.PythonVersions import python_version
 
 from .ExpressionBases import ExpressionChildrenHavingBase
@@ -36,18 +35,15 @@ from .NodeMakingHelpers import (
 class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_EVAL"
 
-    named_children = ("source", "globals", "locals")
-    getSourceCode = ExpressionChildrenHavingBase.childGetter("source")
-    getGlobals = ExpressionChildrenHavingBase.childGetter("globals")
-    getLocals = ExpressionChildrenHavingBase.childGetter("locals")
+    named_children = ("source", "globals_arg", "locals_arg")
 
     def __init__(self, source_code, globals_arg, locals_arg, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values={
                 "source": source_code,
-                "globals": globals_arg,
-                "locals": locals_arg,
+                "globals_arg": globals_arg,
+                "locals_arg": locals_arg,
             },
             source_ref=source_ref,
         )
@@ -58,7 +54,7 @@ class ExpressionBuiltinEval(ExpressionChildrenHavingBase):
 
 
 # Note: Python3 only so far.
-if python_version >= 300:
+if python_version >= 0x300:
 
     class ExpressionBuiltinExec(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXEC"
@@ -79,9 +75,9 @@ if python_version >= 300:
         def computeExpressionDrop(self, statement, trace_collection):
             if self.getParentVariableProvider().isEarlyClosure():
                 result = StatementExec(
-                    source_code=self.getSourceCode(),
-                    globals_arg=self.getGlobals(),
-                    locals_arg=self.getLocals(),
+                    source_code=self.subnode_source,
+                    globals_arg=self.subnode_globals_arg,
+                    locals_arg=self.subnode_locals_arg,
                     source_ref=self.source_ref,
                 )
 
@@ -98,12 +94,12 @@ Replaced built-in exec call to exec statement in early closure context.""",
 
 
 # Note: Python2 only
-if python_version < 300:
+if python_version < 0x300:
 
     class ExpressionBuiltinExecfile(ExpressionBuiltinEval):
         kind = "EXPRESSION_BUILTIN_EXECFILE"
 
-        named_children = ("source", "globals", "locals")
+        named_children = ("source", "globals_arg", "locals_arg")
 
         def __init__(self, source_code, globals_arg, locals_arg, source_ref):
             ExpressionBuiltinEval.__init__(
@@ -121,9 +117,9 @@ if python_version < 300:
 
             if provider.isExpressionClassBody():
                 result = StatementExec(
-                    source_code=self.getSourceCode(),
-                    globals_arg=self.getGlobals(),
-                    locals_arg=self.getLocals(),
+                    source_code=self.subnode_source,
+                    globals_arg=self.subnode_globals_arg,
+                    locals_arg=self.subnode_locals_arg,
                     source_ref=self.source_ref,
                 )
 
@@ -142,30 +138,27 @@ Changed 'execfile' with unused result to 'exec' on class level.""",
 class StatementExec(StatementChildrenHavingBase):
     kind = "STATEMENT_EXEC"
 
-    named_children = ("source", "globals", "locals")
-    getSourceCode = StatementChildrenHavingBase.childGetter("source")
-    getGlobals = StatementChildrenHavingBase.childGetter("globals")
-    getLocals = StatementChildrenHavingBase.childGetter("locals")
+    named_children = ("source", "globals_arg", "locals_arg")
 
     def __init__(self, source_code, globals_arg, locals_arg, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
             values={
-                "globals": globals_arg,
-                "locals": locals_arg,
+                "globals_arg": globals_arg,
+                "locals_arg": locals_arg,
                 "source": source_code,
             },
             source_ref=source_ref,
         )
 
     def setChild(self, name, value):
-        if name in ("globals", "locals"):
+        if name in ("globals_arg", "locals_arg"):
             value = convertNoneConstantToNone(value)
 
         return StatementChildrenHavingBase.setChild(self, name, value)
 
     def computeStatement(self, trace_collection):
-        source_code = trace_collection.onExpression(expression=self.getSourceCode())
+        source_code = trace_collection.onExpression(expression=self.subnode_source)
 
         if source_code.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
@@ -181,7 +174,7 @@ Exec statement raises implicitly when determining source code argument.""",
             )
 
         globals_arg = trace_collection.onExpression(
-            expression=self.getGlobals(), allow_none=True
+            expression=self.subnode_globals_arg, allow_none=True
         )
 
         if globals_arg is not None and globals_arg.mayRaiseException(BaseException):
@@ -200,7 +193,7 @@ Exec statement raises implicitly when determining globals argument.""",
             )
 
         locals_arg = trace_collection.onExpression(
-            expression=self.getLocals(), allow_none=True
+            expression=self.subnode_locals_arg, allow_none=True
         )
 
         if locals_arg is not None and locals_arg.mayRaiseException(BaseException):
@@ -220,16 +213,16 @@ Exec statement raises implicitly when determining locals argument.""",
 
         trace_collection.onExceptionRaiseExit(BaseException)
 
-        str_value = self.getSourceCode().getStrValue()
+        str_value = self.subnode_source.getStrValue()
 
-        if False and str_value is not None:
+        if str_value is not None:
+            # TODO: This needs to be re-done.
             # TODO: Don't forget to consider side effects of source code,
             # locals_arg, and globals_arg.
+            return self, None, None
 
-            # TODO: This needs to be re-done.
-            exec_body = None
-
-            return (exec_body, "new_statements", "In-lined constant exec statement.")
+            # exec_body = ...
+            # return (exec_body, "new_statements", "In-lined constant exec statement.")
 
         return self, None, None
 
@@ -237,12 +230,10 @@ Exec statement raises implicitly when determining locals argument.""",
 class StatementLocalsDictSync(StatementChildHavingBase):
     kind = "STATEMENT_LOCALS_DICT_SYNC"
 
-    named_child = "locals"
-    getLocals = StatementChildHavingBase.childGetter("locals")
+    named_child = "locals_arg"
 
     __slots__ = ("locals_scope", "previous_traces", "variable_traces")
 
-    @calledWithBuiltinArgumentNamesDecorator
     def __init__(self, locals_scope, locals_arg, source_ref):
         StatementChildHavingBase.__init__(self, value=locals_arg, source_ref=source_ref)
 
@@ -287,12 +278,6 @@ class ExpressionBuiltinCompile(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_COMPILE"
 
     named_children = ("source", "filename", "mode", "flags", "dont_inherit", "optimize")
-    getSourceCode = ExpressionChildrenHavingBase.childGetter("source")
-    getFilename = ExpressionChildrenHavingBase.childGetter("filename")
-    getMode = ExpressionChildrenHavingBase.childGetter("mode")
-    getFlags = ExpressionChildrenHavingBase.childGetter("flags")
-    getDontInherit = ExpressionChildrenHavingBase.childGetter("dont_inherit")
-    getOptimize = ExpressionChildrenHavingBase.childGetter("optimize")
 
     def __init__(
         self, source_code, filename, mode, flags, dont_inherit, optimize, source_ref
