@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,12 +29,12 @@ from contextlib import contextmanager
 
 from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
-from nuitka.containers.oset import OrderedSet
+from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
 from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
-from nuitka.nodes.shapes.StandardShapes import tshape_uninit
-from nuitka.tree.SourceReading import readSourceLine
+from nuitka.nodes.shapes.StandardShapes import tshape_uninitialized
+from nuitka.tree.SourceHandling import readSourceLine
 from nuitka.utils.InstanceCounters import (
     counted_del,
     counted_init,
@@ -44,6 +44,9 @@ from nuitka.utils.Timing import TimerReport
 
 from .ValueTraces import (
     ValueTraceAssign,
+    ValueTraceAssignUnescapable,
+    ValueTraceAssignUnescapablePropagated,
+    ValueTraceAssignVeryTrusted,
     ValueTraceDeleted,
     ValueTraceEscaped,
     ValueTraceInit,
@@ -52,7 +55,7 @@ from .ValueTraces import (
     ValueTraceLoopComplete,
     ValueTraceLoopIncomplete,
     ValueTraceMerge,
-    ValueTraceUninit,
+    ValueTraceUninitialized,
     ValueTraceUnknown,
 )
 
@@ -113,7 +116,7 @@ class CollectionUpdateMixin(object):
         return version
 
 
-class CollectionStartpointMixin(CollectionUpdateMixin):
+class CollectionStartPointMixin(CollectionUpdateMixin):
     """Mixin to use in start points of collections.
 
     These are modules, functions, etc. typically entry points.
@@ -253,8 +256,8 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
 
         return trace
 
-    def _initVariableUninit(self, variable):
-        trace = ValueTraceUninit(owner=self.owner, previous=None)
+    def _initVariableUninitialized(self, variable):
+        trace = ValueTraceUninitialized(owner=self.owner, previous=None)
 
         self.addVariableTrace(variable, 0, trace)
 
@@ -296,14 +299,14 @@ class CollectionStartpointMixin(CollectionUpdateMixin):
             # TODO: That's not happening, maybe just assert against it.
             result = self._initVariableInit(variable)
         elif variable.isLocalVariable():
-            result = self._initVariableUninit(variable)
+            result = self._initVariableUninitialized(variable)
         elif variable.isModuleVariable():
             result = self.initVariableModule(variable)
         elif variable.isTempVariable():
-            result = self._initVariableUninit(variable)
+            result = self._initVariableUninitialized(variable)
         elif variable.isLocalsDictVariable():
             if variable.getOwner().getTypeShape() is tshape_dict:
-                result = self._initVariableUninit(variable)
+                result = self._initVariableUninitialized(variable)
             else:
                 result = self.initVariableUnknown(variable)
         else:
@@ -358,7 +361,7 @@ class TraceCollectionBase(object):
     They are kept for "variable" and versions.
     """
 
-    __slots__ = ("owner", "parent", "name", "value_states", "variable_actives")
+    __slots__ = ("owner", "parent", "name", "variable_actives")
 
     if isCountingInstances():
         __del__ = counted_del()
@@ -369,9 +372,6 @@ class TraceCollectionBase(object):
         self.parent = parent
         self.name = name
 
-        # Value state extra information per node.
-        self.value_states = {}
-
         # Currently active values in the tracing.
         self.variable_actives = {}
 
@@ -413,7 +413,7 @@ class TraceCollectionBase(object):
     def markActiveVariableAsEscaped(self, variable):
         current = self.getVariableCurrentTrace(variable)
 
-        if not current.isEscapeOrUnknownOrUninitTrace():
+        if current.isTraceThatNeedsEscape():
             version = variable.allocateTargetNumber()
 
             self.addVariableTrace(
@@ -424,7 +424,7 @@ class TraceCollectionBase(object):
 
             self.markCurrentVariableTrace(variable, version)
 
-    def markActiveVariableAsUnknown(self, variable):
+    def markClosureVariableAsUnknown(self, variable):
         current = self.getVariableCurrentTrace(variable)
 
         if not current.isUnknownTrace():
@@ -438,6 +438,20 @@ class TraceCollectionBase(object):
 
             self.markCurrentVariableTrace(variable, version)
 
+    def markActiveVariableAsUnknown(self, variable):
+        current = self.getVariableCurrentTrace(variable)
+
+        if not current.isUnknownOrVeryTrustedTrace():
+            version = variable.allocateTargetNumber()
+
+            self.addVariableTrace(
+                variable,
+                version,
+                ValueTraceUnknown(owner=self.owner, previous=current),
+            )
+
+            self.markCurrentVariableTrace(variable, version)
+
     def markActiveVariableAsLoopMerge(
         self, loop_node, current, variable, shapes, incomplete
     ):
@@ -449,7 +463,7 @@ class TraceCollectionBase(object):
             # assert shapes, (variable, current)
 
             if not shapes:
-                shapes.add(tshape_uninit)
+                shapes.add(tshape_uninitialized)
 
             result = ValueTraceLoopComplete(loop_node, current, shapes)
 
@@ -510,7 +524,7 @@ class TraceCollectionBase(object):
                     str is not bytes
                     and variable.hasWritersOutsideOf(self.owner) is not False
                 ):
-                    self.markActiveVariableAsUnknown(variable)
+                    self.markClosureVariableAsUnknown(variable)
                 elif variable.hasAccessesOutsideOf(self.owner) is not False:
                     self.markActiveVariableAsEscaped(variable)
 
@@ -526,7 +540,11 @@ class TraceCollectionBase(object):
         pass
 
     def removeAllKnowledge(self):
-        self.markActiveVariablesAsUnknown()
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
+            self.markActiveVariableAsUnknown(variable)
 
     def onVariableSet(self, variable, version, assign_node):
         variable_trace = ValueTraceAssign(
@@ -542,6 +560,51 @@ class TraceCollectionBase(object):
 
         return variable_trace
 
+    def onVariableSetToUnescapableValue(self, variable, version, assign_node):
+        variable_trace = ValueTraceAssignUnescapable(
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable),
+        )
+
+        self.addVariableTrace(variable, version, variable_trace)
+
+        # Make references point to it.
+        self.markCurrentVariableTrace(variable, version)
+
+        return variable_trace
+
+    def onVariableSetToVeryTrustedValue(self, variable, version, assign_node):
+        variable_trace = ValueTraceAssignVeryTrusted(
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable),
+        )
+
+        self.addVariableTrace(variable, version, variable_trace)
+
+        # Make references point to it.
+        self.markCurrentVariableTrace(variable, version)
+
+        return variable_trace
+
+    def onVariableSetToUnescapablePropagatedValue(
+        self, variable, version, assign_node, replacement
+    ):
+        variable_trace = ValueTraceAssignUnescapablePropagated(
+            owner=self.owner,
+            assign_node=assign_node,
+            previous=self.getVariableCurrentTrace(variable),
+            replacement=replacement,
+        )
+
+        self.addVariableTrace(variable, version, variable_trace)
+
+        # Make references point to it.
+        self.markCurrentVariableTrace(variable, version)
+
+        return variable_trace
+
     def onVariableDel(self, variable, version, del_node):
         # Add a new trace, allocating a new version for the variable, and
         # remember the delete of the current
@@ -733,18 +796,21 @@ class TraceCollectionBase(object):
 
         for variable, version in iterItems(collection2.variable_actives):
             if variable not in variable_versions:
-                variable_versions[variable] = 0, version
+                if version != 0:
+                    variable_versions[variable] = 0, version
+                else:
+                    variable_versions[variable] = 0
             else:
                 other = variable_versions[variable]
 
                 if other != version:
                     variable_versions[variable] = other, version
-                else:
-                    variable_versions[variable] = other
 
+        # That would not be fast, pylint: disable=consider-using-dict-items
         for variable in variable_versions:
             if variable not in collection2.variable_actives:
-                variable_versions[variable] = variable_versions[variable], 0
+                if variable_versions[variable] != 0:
+                    variable_versions[variable] = variable_versions[variable], 0
 
         self.variable_actives = {}
 
@@ -907,21 +973,6 @@ class TraceCollectionBase(object):
 
         return new_node, change_tags, message
 
-    def getIteratorNextCount(self, iter_node):
-        return self.value_states.get(iter_node)
-
-    def initIteratorValue(self, iter_node):
-        # TODO: More complex state information will be needed eventually.
-        self.value_states[iter_node] = 0
-
-    def onIteratorNext(self, iter_node):
-        if iter_node in self.value_states:
-            self.value_states[iter_node] += 1
-
-    def resetValueStates(self):
-        for key in self.value_states:
-            self.value_states[key] = None
-
     def addOutlineFunction(self, outline):
         self.parent.addOutlineFunction(outline)
 
@@ -945,13 +996,14 @@ class TraceCollectionBranch(CollectionUpdateMixin, TraceCollectionBase):
         self.variable_traces = parent.variable_traces
 
     def computeBranch(self, branch):
-        if branch.isStatementsSequence():
-            result = branch.computeStatementsSequence(trace_collection=self)
+        assert branch.isStatementsSequence()
 
-            if result is not branch:
-                branch.parent.replaceChild(branch, result)
-        else:
-            self.onExpression(expression=branch)
+        result = branch.computeStatementsSequence(self)
+
+        if result is not branch:
+            branch.parent.replaceChild(branch, result)
+
+        return result
 
     def initVariable(self, variable):
         variable_trace = self.parent.initVariable(variable)
@@ -974,7 +1026,7 @@ class TraceCollectionBranch(CollectionUpdateMixin, TraceCollectionBase):
         Tracing.printSeparator()
 
 
-class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
+class TraceCollectionFunction(CollectionStartPointMixin, TraceCollectionBase):
     __slots__ = (
         "variable_versions",
         "variable_traces",
@@ -994,7 +1046,7 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
             or function_body.isExpressionAsyncgenObjectBody()
         ), function_body
 
-        CollectionStartpointMixin.__init__(self)
+        CollectionStartPointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self,
@@ -1035,7 +1087,7 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
         if locals_scope is not None:
             if not locals_scope.isMarkedForPropagation():
                 for locals_dict_variable in locals_scope.variables.values():
-                    self._initVariableUninit(locals_dict_variable)
+                    self._initVariableUninitialized(locals_dict_variable)
             else:
                 function_body.locals_scope = None
 
@@ -1043,7 +1095,7 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
         trusted_node = self.very_trusted_module_variables.get(variable)
 
         if trusted_node is None:
-            return CollectionStartpointMixin.initVariableModule(self, variable)
+            return CollectionStartPointMixin.initVariableModule(self, variable)
 
         assign_trace = ValueTraceAssign(
             self.owner, assign_node=trusted_node.getParent(), previous=None
@@ -1075,7 +1127,7 @@ class TraceCollectionPureFunction(TraceCollectionFunction):
         TraceCollectionFunction.onUsedFunction(self, function_body=function_body)
 
 
-class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
+class TraceCollectionModule(CollectionStartPointMixin, TraceCollectionBase):
     __slots__ = (
         "variable_versions",
         "variable_traces",
@@ -1090,7 +1142,7 @@ class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
     def __init__(self, module, very_trusted_module_variables):
         assert module.isCompiledPythonModule(), module
 
-        CollectionStartpointMixin.__init__(self)
+        CollectionStartPointMixin.__init__(self)
 
         TraceCollectionBase.__init__(
             self, owner=module, name="module:" + module.getFullName(), parent=None
@@ -1125,7 +1177,7 @@ def areEmptyTraces(variable_traces):
             # from being removed.
 
             return False
-        elif variable_trace.isUninitTrace():
+        elif variable_trace.isUninitializedTrace():
             if variable_trace.getUsageCount():
                 # Checking definite is enough, the merges, we shall see
                 # them as well.
@@ -1167,7 +1219,7 @@ def areReadOnlyTraces(variable_traces):
             # from being not released.
 
             return False
-        elif variable_trace.isUninitTrace():
+        elif variable_trace.isUninitializedTrace():
             pass
         elif variable_trace.isUnknownTrace():
             return False
