@@ -1,4 +1,4 @@
-#     Copyright 2021, Jorj McKie, mailto:<jorj.x.mckie@outlook.de>
+#     Copyright 2022, Jorj McKie, mailto:<jorj.x.mckie@outlook.de>
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -140,7 +140,7 @@ The Tcl library dir. See comments for Tk library dir.""",
         yield self.makeIncludedPackageDataFiles(
             package_name="tkinterdnd2",
             package_directory=module.getCompileTimeDirectory(),
-            pattern=os.path.join("tkinterdnd2", "tkdnd", platform_rep, "**"),
+            pattern=os.path.join("tkdnd", platform_rep, "**"),
             reason="Tcl needed for 'tkinterdnd2' usage",
             tags="tcl",
         )
@@ -258,8 +258,8 @@ The Tcl library dir. See comments for Tk library dir.""",
                 yield self.makeDllEntryPoint(
                     source_path=filename,
                     dest_path=os.path.join("tkinterdnd2", dest_path),
-                    package_name="tkinterdnd2"
-                    # reason="tkinterdnd2 package DLL",
+                    package_name="tkinterdnd2",
+                    reason="tkinterdnd2 package DLL",
                 )
 
 
