@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,11 +24,6 @@ source code comments with Developer Manual sections.
 
 import ast
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeCheck,
     makeExpressionAttributeLookup,
@@ -39,6 +34,8 @@ from nuitka.nodes.ComparisonNodes import makeComparisonExpression
 from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.MatchNodes import ExpressionMatchArgs
+from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
+from nuitka.nodes.ReturnNodes import makeStatementReturnConstant
 from nuitka.nodes.SubscriptNodes import (
     ExpressionSubscriptCheck,
     ExpressionSubscriptLookup,
@@ -48,11 +45,19 @@ from nuitka.nodes.TypeMatchNodes import (
     ExpressionMatchTypeCheckSequence,
 )
 from nuitka.nodes.TypeNodes import ExpressionBuiltinIsinstance
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableNameNodes import StatementAssignmentVariableName
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
+from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
 
 from .ReformulationBooleanExpressions import makeAndNode, makeOrNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
-from .TreeHelpers import buildNode, buildStatementsNode, makeStatementsSequence
+from .TreeHelpers import (
+    buildNode,
+    buildStatementsNode,
+    makeStatementsSequence,
+    makeStatementsSequenceFromStatement,
+)
 
 
 def _makeMatchComparison(left, right, source_ref):
@@ -82,37 +87,69 @@ def _buildCaseBodyCode(provider, case, source_ref):
     return body_code, guard_condition
 
 
-def _buildMatchAs(provider, variable_name, source_value, source_ref):
-    assert "." not in variable_name, variable_name
-    assert "!" not in variable_name, variable_name
+def _buildMatchAs(provider, pattern, make_against, source_ref):
+    variable_name = pattern.name
 
-    return StatementAssignmentVariableName(
-        provider=provider,
-        variable_name=variable_name,
-        source=source_value,
-        source_ref=source_ref,
-    )
+    if variable_name is None:
+        # case _:
+
+        # Assigns to nothing and should be last one in a match
+        # statement, anything after that will be syntax error. TODO: This
+        # ought to only happen once, Python raises it, we do not yet:
+        # SyntaxError: name capture 'var' makes remaining patterns
+        # unreachable
+        assignments = None
+        conditions = None
+    else:
+
+        assert "." not in variable_name, variable_name
+        assert "!" not in variable_name, variable_name
+
+        if pattern.pattern is not None:
+            conditions, assignments = _buildMatch(
+                provider=provider,
+                pattern=pattern.pattern,
+                make_against=make_against,
+                source_ref=source_ref,
+            )
+
+            if assignments is None:
+                assignments = []
+        else:
+            assignments = []
+            conditions = None
+
+        assignments.append(
+            StatementAssignmentVariableName(
+                provider=provider,
+                variable_name=variable_name,
+                source=make_against(),
+                source_ref=source_ref,
+            )
+        )
+
+    return conditions, assignments
 
 
-def _buildMatchValue(provider, against, pattern, source_ref):
+def _buildMatchValue(provider, make_against, pattern, source_ref):
     if type(pattern) is ast.MatchValue:
         right = buildNode(provider, pattern.value, source_ref)
     else:
         right = makeConstantRefNode(constant=pattern.value, source_ref=source_ref)
 
     return _makeMatchComparison(
-        left=against,
+        left=make_against(),
         right=right,
         source_ref=source_ref,
     )
 
 
-def _buildMatchSequence(provider, pattern, against, source_ref):
+def _buildMatchSequence(provider, pattern, make_against, source_ref):
     # Many cases due to recursion, pylint: disable=too-many-locals
 
     conditions = [
         ExpressionMatchTypeCheckSequence(
-            value=against.makeClone(),
+            value=make_against(),
             source_ref=source_ref,
         )
     ]
@@ -137,7 +174,7 @@ def _buildMatchSequence(provider, pattern, against, source_ref):
         conditions.append(
             makeComparisonExpression(
                 left=ExpressionBuiltinLen(
-                    value=against.makeClone(),
+                    value=make_against(),
                     source_ref=source_ref,
                 ),
                 right=makeConstantRefNode(constant=min_length, source_ref=source_ref),
@@ -159,14 +196,14 @@ def _buildMatchSequence(provider, pattern, against, source_ref):
             offset = -(len(pattern.patterns) - count)
 
         if seq_pattern.__class__ is ast.MatchStar:
+            star_pos = count
+
             variable_name = seq_pattern.name
 
             if variable_name is not None:
                 assert "." not in variable_name, variable_name
                 assert "!" not in variable_name, variable_name
 
-                star_pos = count
-
                 # Last one
                 if star_pos == len(pattern.patterns):
                     slice_value = slice(count)
@@ -179,7 +216,7 @@ def _buildMatchSequence(provider, pattern, against, source_ref):
                         variable_name=variable_name,
                         source=ExpressionBuiltinList(
                             ExpressionSubscriptLookup(
-                                expression=against.makeClone(),
+                                expression=make_against(),
                                 subscript=makeConstantRefNode(
                                     constant=slice_value, source_ref=source_ref
                                 ),
@@ -194,8 +231,9 @@ def _buildMatchSequence(provider, pattern, against, source_ref):
             item_conditions, item_assignments = _buildMatch(
                 provider=provider,
                 pattern=seq_pattern,
-                against=ExpressionSubscriptLookup(
-                    expression=against.makeClone(),
+                # It's called before return, pylint: disable=cell-var-from-loop
+                make_against=lambda: ExpressionSubscriptLookup(
+                    expression=make_against(),
                     subscript=makeConstantRefNode(
                         constant=offset, source_ref=source_ref
                     ),
@@ -213,10 +251,10 @@ def _buildMatchSequence(provider, pattern, against, source_ref):
     return conditions, assignments
 
 
-def _buildMatchMapping(provider, pattern, against, source_ref):
+def _buildMatchMapping(provider, pattern, make_against, source_ref):
     conditions = [
         ExpressionMatchTypeCheckMapping(
-            value=against.makeClone(),
+            value=make_against(),
             source_ref=source_ref,
         )
     ]
@@ -230,23 +268,20 @@ def _buildMatchMapping(provider, pattern, against, source_ref):
     for key, kwd_pattern in zip(pattern.keys, pattern.patterns):
         conditions.append(
             ExpressionSubscriptCheck(
-                expression=against.makeClone(),
-                subscript=makeConstantRefNode(
-                    constant=key.value, source_ref=source_ref
-                ),
+                expression=make_against(),
+                subscript=buildNode(provider, key, source_ref),
                 source_ref=source_ref,
             )
         )
 
-        item_against = ExpressionSubscriptLookup(
-            expression=against.makeClone(),
-            subscript=makeConstantRefNode(constant=key.value, source_ref=source_ref),
-            source_ref=source_ref,
-        )
-
+        # It's called before return, pylint: disable=cell-var-from-loop
         item_conditions, item_assignments = _buildMatch(
             provider=provider,
-            against=item_against,
+            make_against=lambda: ExpressionSubscriptLookup(
+                expression=make_against(),
+                subscript=buildNode(provider, key, source_ref),
+                source_ref=source_ref,
+            ),
             pattern=kwd_pattern,
             source_ref=source_ref,
         )
@@ -260,32 +295,40 @@ def _buildMatchMapping(provider, pattern, against, source_ref):
     return conditions, assignments
 
 
-def _buildMatchClass(provider, pattern, against, source_ref):
+def _buildMatchClass(provider, pattern, make_against, source_ref):
     cls_node = buildNode(provider, pattern.cls, source_ref)
 
     assignments = []
 
     conditions = [
         ExpressionBuiltinIsinstance(
-            instance=against.makeClone(),
+            instance=make_against(),
             classes=cls_node,
             source_ref=source_ref,
         )
     ]
 
-    assert not (pattern.patterns and pattern.kwd_patterns), ast.dump(pattern)
-    assert len(pattern.kwd_attrs) == len(pattern.kwd_patterns), ast.dump(pattern)
+    assert not (pattern.patterns and pattern.kwd_patterns), (
+        source_ref,
+        ast.dump(pattern),
+    )
+    assert len(pattern.kwd_attrs) == len(pattern.kwd_patterns), (
+        source_ref,
+        ast.dump(pattern),
+    )
 
     for count, pos_pattern in enumerate(pattern.patterns):
         # TODO: Not reusing Match args, is very wasteful for performance, but
         # temporary variable handling is a bit of a problem in this so far,
         # we should create an outline function for it, but match value args
         # ought to be global probably, so they can be shared.
+
+        # It's called before return, pylint: disable=cell-var-from-loop
         item_conditions, item_assignments = _buildMatch(
             provider=provider,
-            against=ExpressionSubscriptLookup(
+            make_against=lambda: ExpressionSubscriptLookup(
                 expression=ExpressionMatchArgs(
-                    expression=against.makeClone(),
+                    expression=make_against(),
                     max_allowed=len(pattern.patterns),
                     source_ref=source_ref,
                 ),
@@ -305,16 +348,17 @@ def _buildMatchClass(provider, pattern, against, source_ref):
     for key, kwd_pattern in zip(pattern.kwd_attrs, pattern.kwd_patterns):
         conditions.append(
             ExpressionAttributeCheck(
-                expression=against.makeClone(),
+                expression=make_against(),
                 attribute_name=key,
                 source_ref=source_ref,
             )
         )
 
+        # It's called before return, pylint: disable=cell-var-from-loop
         item_conditions, item_assignments = _buildMatch(
             provider=provider,
-            against=makeExpressionAttributeLookup(
-                expression=against.makeClone(),
+            make_against=lambda: makeExpressionAttributeLookup(
+                expression=make_against(),
                 attribute_name=key,
                 source_ref=source_ref,
             ),
@@ -331,74 +375,119 @@ def _buildMatchClass(provider, pattern, against, source_ref):
     return conditions, assignments
 
 
-def _buildMatch(provider, pattern, against, source_ref):
-    if pattern.__class__ is ast.MatchOr:
-        or_condition_list = []
-        for or_pattern in pattern.patterns:
-            or_conditions, or_assignments = _buildMatch(
-                provider=provider,
-                pattern=or_pattern,
-                against=against,
-                source_ref=source_ref,
-            )
-            assert not or_assignments
+def _buildMatchOr(provider, pattern, make_against, source_ref):
+    # Slightly complicated, due to using an outline body, pylint: disable=too-many-locals
 
-            or_condition_list.append(
-                makeAndNode(values=or_conditions, source_ref=source_ref)
-            )
+    or_condition_list = []
+    or_assignments_list = []
 
+    for or_pattern in pattern.patterns:
+        or_conditions, or_assignments = _buildMatch(
+            provider=provider,
+            pattern=or_pattern,
+            make_against=make_against,
+            source_ref=source_ref,
+        )
+
+        or_condition_list.append(
+            makeAndNode(values=or_conditions, source_ref=source_ref)
+        )
+
+        # TODO: Inconsistency somewhere, can return empty.
+        or_assignments_list.append(or_assignments or None)
+
+    if all(or_assignments is None for or_assignments in or_assignments_list):
         condition = makeOrNode(values=or_condition_list, source_ref=source_ref)
         conditions = (condition,)
         assignments = None
+    else:
+        body = None
 
+        for or_condition, or_assignments in zip(
+            reversed(or_condition_list), reversed(or_assignments_list)
+        ):
+            assert or_assignments is not None, (source_ref, or_assignments_list)
+            statements = list(or_assignments)
+            statements.append(
+                makeStatementReturnConstant(constant=True, source_ref=source_ref)
+            )
+
+            if body is None:
+                body = makeStatementConditional(
+                    condition=or_condition,
+                    yes_branch=makeStatementsSequence(
+                        statements=statements, allow_none=False, source_ref=source_ref
+                    ),
+                    no_branch=makeStatementReturnConstant(
+                        constant=False, source_ref=source_ref
+                    ),
+                    source_ref=source_ref,
+                )
+            else:
+                body = makeStatementConditional(
+                    condition=or_condition,
+                    yes_branch=makeStatementsSequence(
+                        statements=statements, allow_none=False, source_ref=source_ref
+                    ),
+                    no_branch=body,
+                    source_ref=source_ref,
+                )
+
+        body = makeStatementsSequenceFromStatement(body)
+
+        outline_body = ExpressionOutlineBody(
+            provider=provider, name="match_or", source_ref=source_ref
+        )
+
+        outline_body.setChild("body", body)
+
+        conditions = (outline_body,)
+        assignments = None
+
+    return conditions, assignments
+
+
+def _buildMatch(provider, pattern, make_against, source_ref):
+    if pattern.__class__ is ast.MatchOr:
+        conditions, assignments = _buildMatchOr(
+            provider=provider,
+            pattern=pattern,
+            make_against=make_against,
+            source_ref=source_ref,
+        )
     elif pattern.__class__ is ast.MatchClass:
         conditions, assignments = _buildMatchClass(
-            provider=provider, pattern=pattern, against=against, source_ref=source_ref
+            provider=provider,
+            pattern=pattern,
+            make_against=make_against,
+            source_ref=source_ref,
         )
     elif pattern.__class__ is ast.MatchMapping:
         conditions, assignments = _buildMatchMapping(
             provider=provider,
             pattern=pattern,
-            against=against,
+            make_against=make_against,
             source_ref=source_ref,
         )
-
     elif pattern.__class__ is ast.MatchSequence:
         conditions, assignments = _buildMatchSequence(
             provider=provider,
             pattern=pattern,
-            against=against,
+            make_against=make_against,
             source_ref=source_ref,
         )
-
     elif pattern.__class__ is ast.MatchAs:
-        conditions = None
-
-        # default match only current with or without a name assigned. TODO: This ought to only
-        # happen once, Python raises it, we do not yet: SyntaxError: name capture 'var' makes
-        # remaining patterns unreachable
-        if pattern.name is None:
-            # case _:
-            # Assigns to nothing and should be last one in a match statement, anything
-            # after that will be syntax error.
-            assignments = None
-        else:
-            # case var:
-            # Assigns to var and should be last one in a match statement, anything
-            # after that will be syntax error.
-            assignment = _buildMatchAs(
-                provider=provider,
-                variable_name=pattern.name,
-                source_value=against,
-                source_ref=source_ref,
-            )
-            assignments = (assignment,)
-
+        conditions, assignments = _buildMatchAs(
+            provider=provider,
+            pattern=pattern,
+            make_against=make_against,
+            source_ref=source_ref,
+        )
     elif pattern.__class__ is ast.MatchValue or pattern.__class__ is ast.MatchSingleton:
         conditions = [
             _buildMatchValue(
                 provider=provider,
-                against=against,
+                make_against=make_against,
                 pattern=pattern,
                 source_ref=source_ref,
             )
@@ -417,12 +506,14 @@ def _buildCase(provider, case, tmp_subject, source_ref):
 
     pattern = case.pattern
 
-    against = ExpressionTempVariableRef(variable=tmp_subject, source_ref=source_ref)
+    make_against = lambda: ExpressionTempVariableRef(
+        variable=tmp_subject, source_ref=source_ref
+    )
 
     conditions, assignments = _buildMatch(
         provider=provider,
         pattern=pattern,
-        against=against,
+        make_against=make_against,
         source_ref=source_ref,
     )
 
@@ -468,7 +559,7 @@ def buildMatchNode(provider, node, source_ref):
             branch_code = makeStatementsSequence(
                 statements=(
                     branch_code,
-                    StatementAssignmentVariable(
+                    makeStatementAssignmentVariable(
                         variable=tmp_indicator_variable,
                         source=makeConstantRefNode(
                             constant=True, source_ref=source_ref
@@ -529,7 +620,7 @@ def buildMatchNode(provider, node, source_ref):
 
     return makeStatementsSequence(
         statements=(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_subject,
                 source=subject_node,
                 source_ref=subject_node.getSourceReference(),
@@ -537,7 +628,7 @@ def buildMatchNode(provider, node, source_ref):
             makeTryFinallyStatement(
                 provider=provider,
                 tried=case_statements,
-                final=StatementReleaseVariable(
+                final=makeStatementReleaseVariable(
                     variable=tmp_indicator_variable, source_ref=source_ref
                 ),
                 source_ref=source_ref,
