@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -62,8 +62,12 @@ class TimerReport(object):
     def __init__(self, message, logger=None, decider=True, min_report_time=None):
         self.message = message
 
+        # Shortcuts.
         if decider is True:
             decider = lambda: True
+        elif decider is False:
+            decider = lambda: False
+
         if logger is None:
             logger = general
 
@@ -80,6 +84,8 @@ class TimerReport(object):
         self.timer = StopWatch()
         self.timer.start()
 
+        return self.timer
+
     def __exit__(self, exception_type, exception_value, exception_tb):
         self.timer.end()
 
